import {wrapperNativeName, safeFunction, required} from './utils/tools';

const win = window;
const doc = document;

// 用 Promise 包裹 sdk
export const sdk = new Promise((resolve, reject) => {
  if (win.platform) {
    resolve(win.platform);
  } else {
    // 添加 sdk 监听事件
    doc.addEventListener('platformReady', () => {
      if (typeof win.platform === 'undefined' || !win.platform) {
        console.log('未获取到 sdk!');
        reject(new Error('未获取到 sdk!'));
      } else {
        resolve(win.platform);
      }
    }, false);
  }
});

/**
 * 平台对外暴露的接口及事件对象
 * @class appSdk
 * {@link http://wiki.cbpmgt.com/confluence/pages/viewpage.action?pageId=19693404 WIKI}
 * demo: http://172.25.47.49/hybrid/demo/platform.html platformDemo
 * http://wiki.cbpmgt.com/confluence/pages/viewpage.action?pageId=20402153
 */

/**
 * 1 env
 * 2 setTitle
 * 3 setTitleBarVisible
 * 4 setMenu
 * 5 getCommonInfo
 * 6 getUserInfo
 * 7 request
 * 8 toast
 * 9 confirm
 * 10 close
 * 11 start
 * 12 checkNetwork
 * 13 share
 * 14 buryPageStart
 * 15 buryPageEnd
 * 16 buryEvent
 * 17 checkLoginStatus
 * 18 putCache
 * 19 getCache
 * 20 removeCache
 * 以下为事件监听
 * 21 onBackPress
 * 22 onClose
 * 23 onLogout
 * 24 onNetChanged
 * 25 onBackground
 * 26 onForeground
 * 27 onShake
 * 28 onPush
 * 直销银行App额外方法
 * 29 getAppCommonInfo
 * 30 appRequest
 * 31 showTradePasswordInput
 * 32 authOutOfDate
 * 33 showResultCtrlDialog
 */
const appSdk = {

  debug: process.env.NODE_ENV === 'development', // 是否启用调试,再调试模式下参数检查情况会输出到控制台

  /**
   * 设置
   * @param debug
   */
  setDebug(debug) {
    this.debug = debug;
  },

  /*eslint-disable max-len*/
  /**
   * 获取platform协议的实现情况，比如可用的事件、可用的接口、协议版本等
   * @param callback
   * 返回的结果
   {
      code: 0,
      data: '*appRequest*authOutOfDate*buryEvent*buryPageEnd*buryPageStart*checkLoginStatus*checkNetwork*close*confirm*getAppCommonInfo*getCache*getCommonInfo*getPayLimit*onEvent*putCache*removeCache*request*setMenu*setTitle*setTitleBarVisible*showResultCtrlDialog*showTradePasswordInput*toast',
      time: '1505728066354'
    }
   */
  env(callback) { // 1
    return sdk.then((platform) => {
      platform.env(callback);
    });
  },

  /**
   * 设置浏览器标题
   * app默认使用页面title字段,手动调用此api会覆盖原有title
   * 只对当前页面有效
   * @param param {Object}
   * * 参数格式
   {
      titleName: '', // String no 标题名称
      titleColor: '', // String no 标题颜色 标准色值比如 #FFFFFF
      titleBackgroundColor: '', //  String no 标题颜色 标准色值比如 #FFFFFF
      subTitleName: '', //  String no 子标题名称
      subTitleColor: '', //  String no 子标题颜色 标准色值比如 #FFFFFF
      titleDivider: '', //  String no  标题下方分割线显示：visible 隐藏：hide 不传：保持不变。 默认显示分割线
      backTitleColor: '', //  String no 返回按钮的颜色 标准色值比如 #FFFFFF
      closeTitleColor: '', //  String no 关闭按钮的颜色 标准色值比如 #FFFFFF
   }
   *
   */
  setTitle(param) { // 2
    if (required(this.debug, {
        param,
      }, 'setTitle')) {

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('setTitle'), param);
      });
    }
  },

  /**
   * 设置标题栏的显示/隐藏
   * @param visible {boolean}
   */
  setTitleBarVisible(visible = false) { // 3
    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('setTitleBarVisible'), {visible});
    });
  },

  /**
   * 设置当前页面头部右侧菜单
   * 单个直接显示在右侧，多个的话以下拉的形式显示
   * 目前实现：Android 当路由切换后会自动去掉右侧菜单
   * 但 IOS 需要手动清除掉右侧菜单，传入空数组即可
   * @param menus {Array}
   * 参数格式
   [{
      menuTitle: "扫一扫",
      menuImage: "http://localhost:8080/sao.png",
      menuAction: "function"
   }]
   *
   */
  setMenu(menus = []) { // 4
    if (!Array.isArray(menus)) {
      if (this.debug) {
        console.log('setMenu, menus 必须是一个数组类型');
      }
      return;
    }
    for (let l = menus.length; l > 0; l--) {
      const _menu = menus[l - 1];
      if (!required(this.debug, {
          [`menus[${l}].menuTitle`]: _menu.menuTitle,
          [`menus[${l}].menuAction`]: _menu.menuAction
        }, 'setMenu')) {
        return;
      }
    }

    menus.forEach((it) => {
      it.menuAction = safeFunction(it.menuAction);
    });

    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('setMenu'), {menus});
    });
  },

  /**
   * 获取公共参数信息
   * @param callback {function}
   * 返回的数据格式为
   * 注意：data 中的值是字符串 json 数据，需要使用 JSON.parse(data) 解析处理
   {
    code: 0,
    time: 1505199918343,
    data: {
      appVersion: 'app 版本',
      channel: '渠道标识',
      clientName: '设备（ios，android',
      deviceId: '设备唯一标识',
      deviceType: '设备类型',
      mac: 'mac地址',
      osVersion: ''
    }
   }
   */
  getCommonInfo(callback) { // 5
    if (required(this.debug, {callback}, 'getCommonInfo')) {
      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('getCommonInfo'), {}, safeFunction(callback));
      });
    }
  },

  /**
   * 获取当前用户的基本信息
   * @param callback {function}
   * 返回的数据格式为
   * 注意：data 中的值是字符串 json 数据，需要使用 JSON.parse(data) 解析处理
   {
    code: 0,
    time: 1505199918343,
    data: {
      channelUserId: '渠道用户标识',
      hasOpenSecAccount: '是否已开通二类户'
    }
   }
   */
  getUserInfo(callback) { // 6
    if (required(this.debug, {callback}, 'getUserInfo')) {
      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('getUserInfo'), {}, safeFunction(callback));
      });
    }
  },

  /**
   * 网络请求
   * @param param {{}}
   * param 格式
   {
    url: '请求url地址',
    httpMethod: 'GET 或 POST DELETE 等', // 注意是大写
    parameters: 'json 字符串' // JSON.stringify(json)
   }
   * @param callback {function}
   */
  request(param, callback) { // 7
    if (required(this.debug, {
        param,
        'param.url': param && param.url,
        'param.httpMethod': param && param.httpMethod,
        'param.parameters': param && param.parameters,
        callback
      }, 'request')) {

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('request'), param, safeFunction(callback));
      });
    }
  },

  /**
   * 弹出toast
   * @param param {{}} { msg(必填):String }
   */
  toast(param) { // 8
    if (required(this.debug, {param, 'param.msg': param && param.msg}, 'toast')) {
      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('toast'), param);
      });
    }
  },

  /**
   * 确认提示框
   * @param param {{}} { title(必填):String, msg(必填):String, cancelBtnText:String, cancelCallBack:function, okBtnText:String, okCallBack:function }
   */
  confirm(param) { // 9
    if (required(this.debug, {
        param,
        'param.title': param && param.title,
        'param.msg': param && param.msg
      }, 'confirm')) {

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('confirm'), param);
      });
    }
  },

  /**
   * 关闭浏览器
   */
  close() { // 10
    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('close'), {});
    });
  },

  /**
   * 启动app模块，目前实现的模块有：登录：LOGIN（如果当前是登录态，则不会跳到登录页面，只会回调 callback）开户：OPENACCOUNT
   * @param param {{}}
   * @param callback {function}
   * 回调结果根据具体的模块返回不同的数据
   * name是LOGIN的情况下：返回值和 getUserInfo 返回值相同
   */
  start(param, callback) { // 11
    if (required(this.debug, {
        param,
        'param.name': param && param.name,
        'param.param': param && param.param
      }, 'start')) {

      param.param = JSON.stringify(param.param);

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('start'), param, safeFunction(callback));
      });

    }
  },

  /**
   * 检查设备的网络状况
   *
   * @param callback {function} 回调参数 : { hasNetwork:boolean, netType:String(wifi|2G|3G|4G|unknow) }
   */
  checkNetwork(callback) { // 12
    if (required(this.debug, {callback}, 'checkNetwork')) {

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('checkNetwork'), {}, (resp) => {
          if (resp && resp.code === 0 && resp.data) {
            resp.data.hasNetwork = Boolean(resp.data.hasNetwork);
          }
          callback(resp);
        });
      });
    }
  },

  /**
   * 调用app的分享功能
   * @param param {{}} { type(必填): String[text|web|view], channel(必填): String[WX|QQ|WB|MON|STOCK] }
   * * 参数格式
   {
      channel: 'WX|WB|QQ|MON|STOCK', // 分享渠道， WX|WB|QQ|MON|STOCK 中间用竖线隔开，可选其中某个或某几个， 空串的话默认为全部。WX代表微信好友和微信朋友圈
      url: '', // Web分享地址
      title: '', //  Web分享标题
      desc: '', //  Web分享描述
      text: '', //  文本分享内容
      iconUrl: '', //  Web分享图标
      startY: '', //  截图起始 0-1的值 页面比例
      endY: '', //  截图尾部 0-1的值 页面比例
      type: '', // 分享类型（text, view, web）(类型为text时，text必填类型，为view时，startY和endY必填类型，为web时，url必填)
   }
   Fixme 目前该功能，手机端实现有问题
   * @param callback {function}
   */
  share(param, callback) { // 13
    if (required(this.debug, {
        param,
        'param.type': param && param.type,
        'param.channel': param && param.channel
      }, 'share')) {

      if (['text', 'web', 'view'].indexOf(param.type) === -1) {
        if (this.debug) {
          console.log('不正确的分享类型:', param.type);
        }
        return;
      }

      const channels = param.channel.split('|'),
        expectChannel = ['WX', 'QQ', 'WB', 'MON', 'STOCK'];
      for (let l = channels.length; l > 0; l--) {
        if (expectChannel.indexOf(channels[l - 1]) === -1) {
          if (this.debug) {
            console.log('不正确的分享渠道:', param.channel);
          }
          return;
        }
      }

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('share'), param, safeFunction(callback));
      });

    }
  },

  /**
   * 进入页面埋点
   * @param param {page: 'page1'} 页面名称 (同一页面的页面名称应和 buryPageEnd 中的页面名称相同)
   * @returns {Promise.<TResult>}
   */
  buryPageStart(param) { // 14
    if (required(this.debug, {
        param,
      }, 'buryPageStart')) {

      return sdk.then((platform) => {
        platform.on(wrapperNativeName('buryPageStart'), param);
      });

    }
  },

  /**
   * 离开页面埋点
   * @param param {page: 'page1'} 页面名称 (同一页面的页面名称应和 buryPageStart 中的页面名称相同)
   * @returns {Promise.<TResult>}
   */
  buryPageEnd(param) { // 15
    if (required(this.debug, {
        param,
      }, 'buryPageEnd')) {

      return sdk.then((platform) => {
        platform.on(wrapperNativeName('buryPageEnd'), param);
      });

    }
  },

  /**
   * 埋点事件
   * @param param {}
   * 传参格式
   {
     eventID: '', //事件ID
     properties: {} // 事件属性
   }
   * @returns {Promise.<TResult>}
   */
  buryEvent(param) { // 16
    if (required(this.debug, {
        param
      }, 'buryEvent')) {

      return sdk.then((platform) => {
        platform.on(wrapperNativeName('buryEvent'), param);
      });
    }
  },

  /**
   * 检查是否是登录状态
   * @param callback
   * 返回结果
   {
     code: 0,
     data: {
      isLogin: true // 是否为登录态
     },
     time: ''
   }
   * @returns {Promise.<TResult>}
   */
  checkLoginStatus(callback) { // 17
    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('checkLoginStatus'), {}, safeFunction(callback));
    });
  },

  /**
   * 放入本地缓存,考虑缓冲区已满的问题
   * @param {string} type local|session
   * @param key {String}
   * @param value {String}
   * @param callback {function}
   */
  putCache({type, key, value}, callback) { // 18
    if (!type || typeof key !== 'string') {
      return;
    }

    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('putCache'), {type, key, value}, safeFunction(callback));
    }).catch((e) => {
      if (this.debug) {
        console.log(type, 'storage is full.');
      }
    });
  },


  /**
   * 从本地缓冲区获取数据,可以一次取多个key,以spliter分割,默认为';'
   * @param type {String} local|session
   * @param key {String}
   * @returns {{}} json
   * @param callback {function}
   */
  getCache({type, key}, callback) { // 19
    if (!type || typeof key !== 'string') {
      return;
    }
    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('getCache'), {type, key}, callback);
    });
  },

  /**
   * 从缓冲区移除数据,可以一次删除多个key,以spliter分割,默认为';'
   * @param type {String} local|session
   * @param key {String}
   * @param callback {function}
   */
  removeCache({type, key}, callback) { // 20
    if (!type || typeof key !== 'string') {
      return;
    }
    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('removeCache'), {key}, callback);
    });
  },

  /************ 事件监听 **************/
  /**
   * app返回键监听
   * @param callback
   */
  onBackPress(callback) { // 21
    if (required(this.debug, {
        callback
      }, 'onBackPress')) {
      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onBackPress'), {}, safeFunction(callback));
      });
    }
  },
  /**
   * webview关闭监听
   * @param callback
   */
  onClose(callback) { // 22
    if (required(this.debug, {
        callback
      }, 'onClose')) {

      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onClose'), {}, safeFunction(callback));
      });

    }
  },
  /**
   * 当前用户被登出监听
   * @param callback
   */
  onLogout(callback) { // 23
    if (required(this.debug, {
        callback
      }, 'onLogout')) {

      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onLogout'), {}, safeFunction(callback));
      });
    }
  },
  /**
   * 设备网络情况监听
   * @param callback
   */
  onNetChanged(callback) { // 24
    if (required(this.debug, {
        callback
      }, 'onNetChanged')) {
      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onNetChanged'), {}, (resp) => {
          if (resp && resp.code === 0 && resp.data) {
            resp.data.hasNetwork = Boolean(resp.data.hasNetwork);
          }
          callback(resp);
        });
      });
    }
  },
  /**
   * webview切到后台监听
   * @param callback
   */
  onBackground(callback) { // 25
    if (required(this.debug, {
        callback
      }, 'onBackground')) {
      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onBackground'), {}, callback);
      });
    }
  },
  /**
   * webview回到前台监听
   * 只有先触发onBackground才能触发onForeground
   * @param callback
   */
  onForeground(callback) { // 26
    if (required(this.debug, {
        callback
      }, 'onForeground')) {
      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onForeground'), {}, callback);
      });
    }
  },
  /**
   * 设备摇一摇监听
   * @param callback
   */
  onShake(callback) { // 27
    if (required(this.debug, {
        callback
      }, 'onShake')) {
      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onShake'), {}, callback);
      });
    }
  },
  /**
   * app推送消息监听
   * @param busisessType 指定监听的业务类型
   * @param callback
   */
  onPush(busisessType, callback) { // 28
    if (required(this.debug, {
        busisessType,
        callback
      }, 'onPush')) {

      return sdk.then((platform) => {
        platform.on(wrapperNativeName('onPush'), {busisessType}, callback);
      });

    }
  },

  // 以下为直销银行App额外方法

  /**
   * 获取 App 公共参数信息
   * @param callback {function}
   * 返回的数据格式为
   * 注意：data 中的值是字符串 json 数据，需要使用 JSON.parse(data) 解析处理
   *
   {
    code: 0,
    time: 1505199918343,
    data: {
      appVersion: 'app 版本',
      auth: '753b72ef82faa1ca96a9137bfcf5dbd0',
      channel: '渠道标识',
      clientName: '设备（ios，android',
      custId: '101353543487838019584',
      deviceId: '设备唯一标识',
      deviceType: '设备类型',
      deviceName: '设备名称',
      mac: 'mac地址'
    }
   }
   */
  getAppCommonInfo(callback) { // 29
    if (required(this.debug, {callback}, 'getAppCommonInfo')) {
      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('getAppCommonInfo'), {}, safeFunction(callback));
      });
    }
  },

  /**
   * 自定义的非标准SaaS服务的网络请求
   * @param param {{}}
   * param 格式
   {
    url: '请求url地址',
    httpMethod: 'GET 或 POST DELETE 等', // 注意是大写
    parameters: 'json 字符串' // JSON.stringify(json)
   }
   * @param callback {function}
   */
  appRequest(param, callback) { // 30
    if (required(this.debug, {
        param,
        'param.url': param && param.url,
        'param.httpMethod': param && param.httpMethod,
        'param.parameters': param && param.parameters,
        callback
      }, 'request')) {

      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('request'), param, safeFunction(callback));
      });
    }
  },

  /**
   * 调用密码键盘
   * 调起交易密码输入框（专供App使用）
   * @param callback {function}
   * * 返回的数据格式为
   *
   {
    code: 0,
    time: 1505199918343,
    data: {
      timeString: '',
      password: '加密后的密码'
    }
   }
   */
  showTradePasswordInput(callback) { // 31
    if (required(this.debug, {callback}, 'showTradePasswordInput')) {
      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('showTradePasswordInput'), {}, safeFunction(callback));
      });
    }
  },

  /**
   * 登录超时或者单点登录 auth 失效
   */
  authOutOfDate() { // 32
    return sdk.then((platform) => {
      platform.invoke(wrapperNativeName('authOutOfDate'), {});
    });
  },

  /**
   * 弹出ResultControl对话框
   * @param callback {function}
   */
  showResultCtrlDialog(callback) { // 33
    if (required(this.debug, {callback}, 'showResultCtrlDialog')) {
      return sdk.then((platform) => {
        platform.invoke(wrapperNativeName('showResultCtrlDialog'), safeFunction(callback));
      });
    }
  },
};

export default appSdk;
